#!/usr/bin/perl

#┌─────────────────────────────────
#│ HONEY BOARD : post.cgi - 2019/12/22
#│ copyright (c) kentweb, 1997-2019
#│ http://www.kent-web.com/
#└─────────────────────────────────
# REBUILD(改造)
#┌─────────────────────────────────
#│  Honey Board v6.1 PLUS : post.cgi - 2019/12/23
#│  Edited by TRANSFORM Since 2012
#│  https://www.transformsite.com/
#└─────────────────────────────────

# モジュール宣言
use strict;
use CGI::Carp qw(fatalsToBrowser);
use lib "./lib";
use CGI::Minimal;

# 設定ファイル認識
require "./init.cgi";
my %cf = set_init();

# データ受理
CGI::Minimal::max_read_size($cf{maxdata});
my $cgi = CGI::Minimal->new;
error('容量オーバー') if ($cgi->truncated);
my %in = parse_form($cgi);

# アクセス制限
&passwd(%in) if ($cf{enter_pwd} ne '');

# 条件分岐
if ($in{mode} eq 'post') { post_data(); }
if ($in{mode} eq 'dele') { dele_data(); }
if ($in{mode} eq 'edit') { edit_data(); }
error("不明な処理です");

#-----------------------------------------------------------
#  記事投稿
#-----------------------------------------------------------
sub post_data {
	# 投稿チェック
	if ($cf{postonly} && $ENV{REQUEST_METHOD} ne 'POST') {
		error("不正なリクエストです");
	}

	# フォーム入力チェック
	form_check();

	# 画像認証チェック
	if ($cf{use_captcha} > 0) {
		require $cf{captcha_pl};
		if ($in{captcha} !~ /^\d{$cf{cap_len}}$/) {
			error("画像認証が入力不備です。<br />投稿フォームに戻って再読込み後、再入力してください");
		}

		# 投稿キーチェック
		# -1 : キー不一致
		#  0 : 制限時間オーバー
		#  1 : キー一致
		my $chk = cap::check($in{captcha},$in{str_crypt},$cf{captcha_key},$cf{cap_time},$cf{cap_len});
		if ($chk == 0) {
			error("画像認証が制限時間を超過しました。<br />投稿フォームに戻って再読込み後、指定の数字を再入力してください");
		} elsif ($chk == -1) {
			error("画像認証が不正です。<br />投稿フォームに戻って再読込み後、再入力してください");
		}
	}

	# ホスト取得
	my ($host,$addr) = get_host();

	# 時間取得
	my ($date,$times) = get_time();

	# アイコン
	my $icon = (split(/,/, $cf{icon}->[$in{icon}]))[0];

	# 文字色
	my $color = (split(/\s+/, $cf{colors}))[$in{color}];

	open(DAT,"+< $cf{logfile}") or error("open err: $cf{logfile}");
	eval "flock(DAT, 2);";

	# 記事NO処理
	my $top = <DAT>;
	my ($no,$ip,$time2) = split(/<>/,$top);
	$no++;

	# 連続投稿チェック
	if ($addr eq $ip && $cf{wait} > $times - $time2) {
		close(DAT);
		error("連続投稿はもうしばらく時間をおいて下さい");
	}

	# YouTubeタグに改行がある場合は排除
	$in{youtube} =~ s/\n//g;

	# ファイル添付処理
	my ($ext,$w,$h) = upload($no) if ($in{upfile});

	# 暗証キー暗号化
	my $crypt = encrypt($in{pwd}) if ($in{pwd} ne "");

	# 親記事の場合
	my @old;
	if ($in{reno} eq "") {

		# 最大記事数処理
		my ($i,@data);
		while(<DAT>) {
			my ($no,$res) = split(/<>/);

			if (!$res) { $i++; }
				if ($i <= $cf{maxlog} - 1) {
					push(@data,$_);
				} else {
					push(@old,$_);
			}
		}

		unshift(@data,"$no<><>$date<>$in{name}<>$in{email}<>$in{sub}<>$icon<><>$in{comment}<>$color<>$in{url}<>$host<>$crypt<>$ext<>$w<>$h<>$in{col3}<>$in{youtube}<>\n");
		unshift(@data,"$no<>$addr<>$times<>\n");

		# 更新
		seek(DAT, 0, 0);
		print DAT @data;
		truncate(DAT, tell(DAT));
		close(DAT);

	# レス記事の場合：トップソートあり
	} elsif ($in{reno} && $cf{top_sort}) {

		my ($flg,$match,@data,@tmp);
		while(<DAT>) {
			my ($no2,$reno2) = split(/<>/);

			if ($in{reno} eq $no2) {
				if ($reno2) { $flg++; last; }
				$match = 1;
				push(@data,$_);

			} elsif ($in{reno} eq $reno2) {
				push(@data,$_);

			} elsif ($match == 1 && $in{reno} ne $reno2) {
				$match = 2;
				push(@data,"$no<>$in{reno}<>$date<>$in{name}<>$in{email}<>$in{sub}<>$icon<><>$in{comment}<>$color<>$in{url}<>$host<>$crypt<>$ext<>$w<>$h<>$in{col3}<>$in{youtube}<>\n");
				push(@tmp,$_);

			} else {
				push(@tmp,$_);
			}
		}
		if ($flg) {
			close(DAT);
			error("不正な返信要求です");
		}

		if ($match == 1) {
			push(@data,"$no<>$in{reno}<>$date<>$in{name}<>$in{email}<>$in{sub}<>$icon<><>$in{comment}<>$color<>$in{url}<>$host<>$crypt<>$ext<>$w<>$h<>$in{col3}<>$in{youtube}<>\n");
		}
		push(@data,@tmp);

		# 更新
		unshift(@data,"$no<>$addr<>$times<>\n");
		seek(DAT, 0, 0);
		print DAT @data;
		truncate(DAT, tell(DAT));
		close(DAT);

	# レス記事の場合：トップソートなし
	} else {

		my ($flg,$match,@data);
		while(<DAT>) {
			my ($no2,$reno2) = split(/<>/);

			if ($match == 0 && $in{reno} eq $no2) {
				if ($reno2) { $flg++; last; }
				$match = 1;

			} elsif ($match == 1 && $in{reno} ne $reno2) {
				$match = 2;
				push(@data,"$no<>$in{reno}<>$date<>$in{name}<>$in{email}<>$in{sub}<>$icon<><>$in{comment}<>$color<>$in{url}<>$host<>$crypt<>$ext<>$w<>$h<>$in{col3}<>$in{youtube}<>\n");
			}
			push(@data,$_);
		}
		if ($flg) {
			close(DAT);
			error("不正な返信要求です");
		}

		if ($match == 1) {
			push(@data,"$no<>$in{reno}<>$date<>$in{name}<>$in{email}<>$in{sub}<>$icon<><>$in{comment}<>$color<>$in{url}<>$host<>$crypt<>$ext<>$w<>$h<>$in{col3}<>$in{youtube}<>\n");
		}

		# 更新
		unshift(@data,"$no<>$addr<>$times<>\n");
		seek(DAT, 0, 0);
		print DAT @data;
		truncate(DAT, tell(DAT));
		close(DAT);
	}

	# 過去ログ更新
	if ($cf{pastkey} && @old > 0) {
		make_past(@old)
	}

	# メール通知
	my $clip = "$no$ext" if ($ext);
	mail_to($date,$host,$clip) if ($cf{mailing} == 1);

	# クッキー格納
	set_cookie($in{name},$in{email},$in{url},$in{icon},$in{color},$in{col3});

	# 完了メッセージ
	message("投稿は正常に処理されました");
}

#-----------------------------------------------------------
#  過去ログ生成
#-----------------------------------------------------------
sub make_past {
	my @past = @_;

	# 過去ログNOファイル
	open(NO,"+< $cf{nofile}") or error("open err: $cf{nofile}");
	eval "flock(NO, 2);";
	my $num = <NO>;

	# 過去ログを定義
	my $pastfile = "$cf{pastdir}/" . sprintf("%04d",$num) . ".cgi";

	# 過去ログを開く
	open(LOG,"+< $pastfile") or error("open err: $pastfile");
	eval "flock(LOG, 2);";
	my @data = <LOG>;

	# 規定行数オーバー時、次ファイル生成
	if (@data > $cf{pastmax}) {

		# 過去ログを閉じる
		@data = ();
		close(LOG);

		# 過去NO更新
		seek(NO, 0, 0);
		print NO ++$num;
		truncate(NO, tell(NO));
		close(NO);

		$pastfile = "$cf{pastdir}/" . sprintf("%04d",$num) . ".cgi";

		open(LOG,"+> $pastfile");
		eval "flock(LOG, 2);";
		print LOG @past;
		close(LOG);

		chmod(0666, $pastfile);

	# 規定内
	} else {

		# 過去NOファイルを閉じる
		close(NO);

		# 過去ログ更新
		seek(LOG, 0, 0);
		print LOG @past;
		print LOG @data;
		truncate(LOG, tell(LOG));
		close(LOG);
	}
}

#-----------------------------------------------------------
#  ユーザ記事削除
#-----------------------------------------------------------
sub dele_data {
	# 投稿チェック
	if ($cf{postonly} && $ENV{REQUEST_METHOD} ne 'POST') {
		error("不正なリクエストです");
	}

	$in{del} =~ s/\D//g;
	if ($in{del} eq '' or $in{pwd} eq '') {
		error("暗証キーの入力モレ又は記事番号が不明です");
	}

	my ($top,$flg,$oya_flg,@data,$ext);
	open(DAT,"+< $cf{logfile}") or error("open err: $cf{logfile}");
	eval "flock(DAT, 2);";
	my $top = <DAT>;
	while (<DAT>) {
		my ($no,$reno,$date,$name,$eml,$sub,$ico,$ico2,$com,$col,$url,$host,$pw,$ex,$w,$h,$col3) = split(/<>/);

		# 該当記事
		if ($in{del} == $no) {
			$ext = $ex;
			$oya_flg = 1;
			$flg = 1;
			if ($pw eq '') {
				$flg = 2;
				last;
			}
			# 暗証キー照合
			if (decrypt($in{pwd},$pw) != 1) {
				$flg = 3;
				last;
			}
			next;

		# 親記事を削除した場合はレス記事削除
		} elsif ($oya_flg && $in{del} == $reno) {
			next;
		}
		push(@data,$_);
	}
	if (!$flg) {
		close(DAT);
		error("該当記事が見当たりません");
	} elsif ($flg == 2) {
		close(DAT);
		error("該当記事には暗証キーが設定されていません");
	} elsif ($flg == 3) {
		close(DAT);
		error("暗証キーが違います");
	}

	# 更新
	seek(DAT, 0, 0);
	print DAT $top;
	print DAT @data;
	truncate(DAT, tell(DAT));
	close(DAT);

	# 添付削除
	if ($ext) {
		unlink("$cf{imgdir}/$in{del}$ext");
		unlink("$cf{imgdir}/$in{del}-s$ext") if (-f "$cf{imgdir}/$in{del}-s$ext");
	}

	# 完了
	message("記事を削除しました");
}

#-----------------------------------------------------------
#  ユーザ記事修正
#-----------------------------------------------------------
sub edit_data {
	# 投稿チェック
	if ($cf{postonly} && $ENV{REQUEST_METHOD} ne 'POST') {
		error("不正なリクエストです");
	}

	# 入力チェック
	if ($in{edit} eq '' || $in{pwd} eq '') {
		error("記事No又は暗証キーが入力されていません");
	}

	# 実行
	if ($in{job} eq "edit") {

	# フォーム入力チェック
	form_check();

		# アイコン
		my $icon = (split(/,/, $cf{icon}->[$in{icon}]))[0];

		# 文字色
		my $color = (split(/\s+/,$cf{colors}))[$in{color}];

	# 差し替え
	my ($flg,@data,$ext2,$w2,$h2);
	open(DAT,"+< $cf{logfile}") or error("open err: $cf{logfile}");
	eval "flock(DAT, 2);";
	my $top = <DAT>;
	while (<DAT>) {
		my ($no,$reno,$date,$name,$eml,$sub,$ico,$ico2,$com,$col,$url,$host,$pw,$ext,$w,$h,$col3,$tube) = split(/<>/);

			if ($in{edit} == $no) {
				$flg = 1;

			# アップロード
			if ($in{upfile}) {
				($ext2,$w2,$h2) = &upload($in{edit});
			}

			# 画像削除
			if ($in{imgdel}) {
				unlink("$cf{upldir}/$in{edit}$ext");
				$ext = $w = $h = '';
			}

			# 添付アップロードの場合
			if ($ext2) {
				# 拡張子変更の場合旧ファイル削除
				if ($ext && $ext2 ne $ext) {
					unlink("$cf{upldir}/$in{edit}$ext");
				}
				# 新添付ファイル情報
				($ext,$w,$h) = ($ext2,$w2,$h2);
			}

		        	$_ = "$no<>$reno<>$date<>$in{name}<>$in{email}<>$in{sub}<>$icon<>$ico2<>$in{comment}<>$color<>$in{url}<>$host<>$pw<>$ext<>$w<>$h<>$in{col3}<>$in{youtube}<>\n";
			}
			push(@data,$_);
		}

		if ($flg < 1) {
			close(DAT);
			error("不正な処理です");
		}

		# 更新
		seek(DAT, 0, 0);
		print DAT $top;
		print DAT @data;
		truncate(DAT, tell(DAT));
		close(DAT);

		# 完了メッセージ
		message("記事の修正を完了しました");
	}

	# 記事抽出
	my ($flg,$log);
	open(IN,"$cf{logfile}") || &error("Open Error: $cf{logfile}");
	my $top = <IN>;
	while (<IN>) {
		my ($no,$reno,$date,$name,$eml,$sub,$ico,$ico2,$com,$col,$url,$host,$pw,$ex,$w,$h,$col3,$tube) = split(/<>/);

		if ($in{edit} == $no) {

			$flg = 1;
			if ($pw eq '') {
				$flg = -1;
			}
			# 暗証キー照合
			if (decrypt($in{pwd},$pw) != 1) {
				$flg = -2;
			}
			$log = $_;
			last;
		}
	}
	close(IN);

	# 判定
	if (!$flg) {
		error("該当記事が見当たりません");
	} elsif ($flg == -1) {
		error("該当記事には暗証キーが設定されていません");
	} elsif ($flg == -2) {
		error("暗証キーが違います");
	}

	# 改行復元
	my ($no,$reno,$date,$name,$eml,$sub,$icon,$ico2,$com,$col,$url,$host,$pw,$ex,$w,$h,$col3,$tube) = split(/<>/);
	$com =~ s/<br>/\n/g;
	$url ||= 'http://';

	# アイコン
	my %op;
	foreach (0 .. $#{$cf{icon}}) {
		my ($file,$name) = split(/,/, $cf{icon}->[$_]);
		if ($icon eq $file) {
			$op{icon} .= qq|<option value="$_" selected>$name\n|;
		} else {
			$op{icon} .= qq|<option value="$_">$name\n|;
		}
	}

	# 色選択ボタン
	my @col = split(/\s+/, $cf{colors});
	my $color;
	foreach (0 .. $#col) {
		if ($col[$_] eq $col) {
			$color .= qq|<input type="radio" name="color" value="$_" checked>|;
		} else {
			$color .= qq|<input type="radio" name="color" value="$_">|;
		}
		$color .= qq|<span style="color:$col[$_]">■</span>\n|;
	}

	# 枠色プルダウン
        my @col3 = split(/\s+/, $cf{colors_b});
        my @col4 = split(/\s+/, $cf{colors_b2});
	my $color_b;
	foreach (0 .. $#col3) {
		if ($col3[$_] eq $col3) {
			$color_b .= qq|<option value="$col3[$_]" style="color:$col3[$_]" selected>$col4[$_]</option>\n|;
		} else {
			$color_b .= qq|<option value="$col3[$_]" style="color:$col3[$_]">$col4[$_]</option>\n|;
		}
	}

	# 添付ファイル
	my $attach;
	if ($ex) {
		$attach .= qq|&nbsp; [<a href="$cf{upldir}/$no$ex" target="_blank">添付</a>]\n|;
		$attach .= qq|<input type="checkbox" name="imgdel" value="1">削除\n|;
	}

	# 修正フォーム
	open(IN,"$cf{tmpldir}/edit.html") or &error("open err: edit.html");
	my $tmpl = join('', <IN>);
	close(IN);

	if ($in{bbs} == 1) {
		$tmpl =~ s/!bbs_cgi!/$cf{bbs_cgi}?bbs=1/g;
	} else {
		$tmpl =~ s/!bbs_cgi!/$cf{bbs_cgi}/g;
	}


	# 返信記事時除外
	if ($reno) {
		$tmpl =~ s/<!-- res_begin -->.+<!-- res_end -->//gs;
	}

		$tmpl =~ s/!([a-z]+_cgi)!/$cf{$1}/g;
		$tmpl =~ s/!bbs_title!/$cf{bbs_title}/g;
		$tmpl =~ s|!bbs_css!|$cf{cmnurl}/honey.css|g;
		$tmpl =~ s|!mob_css!|$cf{cmnurl}/mobile_honey.css|g;
		$tmpl =~ s|!bbs_js!|$cf{cmnurl}/bbs.js|g;
		$tmpl =~ s/!edit!/$no/g;
		$tmpl =~ s/!pwd!/$pw/g;
		$tmpl =~ s/!name!/$name/g;
		$tmpl =~ s/!email!/$eml/g;
		$tmpl =~ s/!sub!/$sub/g;
		$tmpl =~ s/!comment!/$com/g;
		$tmpl =~ s/!url!/$url/g;
		$tmpl =~ s/!color!/$color/g;
		$tmpl =~ s/!color3!/$col3/g;
		$tmpl =~ s/!col!/$col/g;
		$tmpl =~ s|!icon!|<img src="$cf{imgurl}/$icon" alt="" />|g;
		$tmpl =~ s/<!-- op_([a-z]+) -->/$op{$1}/g;
		$tmpl =~ s/<!-- op_color2 -->/$color_b/g;
		$tmpl =~ s/<!-- attach -->/$attach/g;
		$tmpl =~ s/<!-- youtube -->/$tube/;

	print "Content-type: text/html; charset=utf-8\n\n";
	print $tmpl;

	# フッタ
	footer();

	exit;
}

#-----------------------------------------------------------
#  メール送信
#-----------------------------------------------------------
sub mail_to {
	my ($date,$host,$clip) = @_;

	# 件名をMIMEエンコード
	require './lib/jacode.pl';
	my $msub = mime_unstructured_header("BBS : $in{sub}");

	# コメント内の改行復元
	my $com = $in{comment};
	$com =~ s/<br>/\n/g;
	$com =~ s/&lt;/>/g;
	$com =~ s/&gt;/</g;
	$com =~ s/&quot;/"/g;
	$com =~ s/&amp;/&/g;
	$com =~ s/&#39;/'/g;

	# メール本文を定義
	my $body = <<EOM;
掲示板に投稿がありました。

投稿日：$date
ホスト：$host

件名  ：$in{sub}
お名前：$in{name}
E-mail：$in{email}
URL   ：$in{url}
添付：$clip

$com
EOM

	# JISコード変換
	my $mbody;
	for my $tmp ( split(/\n/,$body) ) {
		jcode::convert(\$tmp,'jis','utf8');
		$mbody .= "$tmp\n";
	}

	# FROM行の定義
	my $from = $in{email} || $cf{mailto};
	
	# sendmailコマンド
	my $scmd = "$cf{sendmail} -t -i";
	if ($cf{sendm_f} == 1) { $scmd .= " -f $from"; }

	# 送信
	open(MAIL,"| $scmd") or error("送信失敗");
	print MAIL "To: $cf{mailto}\n";
	print MAIL "From: $from\n";
	print MAIL "Subject: $msub\n";
	print MAIL "MIME-Version: 1.0\n";
	print MAIL "Content-type: text/plain; charset=ISO-2022-JP\n";
	print MAIL "Content-Transfer-Encoding: 7bit\n";
	print MAIL "X-Mailer: $cf{version}\n\n";
	print MAIL "$mbody\n";
	close(MAIL);
}

#-----------------------------------------------------------
#  禁止ワードチェック
#-----------------------------------------------------------
sub no_wd {
	my $flg;
	foreach ( split(/,/, $cf{no_wd}) ) {
		if (index("$in{name} $in{sub} $in{comment}", $_) >= 0) {
			$flg = 1;
			last;
		}
	}
	if ($flg) { error("禁止ワードが含まれています"); }
}

#-----------------------------------------------------------
#  日本語チェック
#-----------------------------------------------------------
sub jp_wd {
	if ($in{comment} !~ /(?:[\xC0-\xDF][\x80-\xBF]|[\xE0-\xEF][\x80-\xBF]{2}|[\xF0-\xF7][\x80-\xBF]{3})/x) {
		error("メッセージに日本語が含まれていません");
	}
}

#-----------------------------------------------------------
#  URL個数チェック
#-----------------------------------------------------------
sub urlnum {
	my $com = $in{comment};
	my ($num) = ($com =~ s|(https?://)|$1|ig);
	if ($num > $cf{urlnum}) {
		error("コメント中のURLアドレスは最大$cf{urlnum}個までです");
	}
}

#-----------------------------------------------------------
#  完了メッセージ
#-----------------------------------------------------------
sub message {
	my ($msg) = @_;

	open(IN,"$cf{tmpldir}/message.html") or error("open err: message.html");
	my $tmpl = join('', <IN>);
	close(IN);

	$tmpl =~ s/!bbs_cgi!/$cf{bbs_cgi}/g;
	$tmpl =~ s|!bbs_css!|$cf{cmnurl}/honey.css|g;
	$tmpl =~ s|!mob_css!|$cf{cmnurl}/mobile_honey.css|g;
	$tmpl =~ s|!bbs_js!|$cf{cmnurl}/bbs.js|g;
	$tmpl =~ s/!bbs_title!/$cf{bbs_title}/g;
	$tmpl =~ s/!message!/$msg/g;
	#$tmpl =~ s/!bbs!/$in{bbs} ? 1 : 0/e;

	if ($in{bbs} == 1) {
		$tmpl =~ s|<!-- hidden -->|<input type="hidden" name="bbs" value="1" />|g;
	}

	print "Content-type: text/html; charset=utf-8\n\n";
	print $tmpl;
	exit;
}

#-----------------------------------------------------------
#  ホスト名取得
#-----------------------------------------------------------
sub get_host {
	# IP&ホスト取得
	my $host = $ENV{REMOTE_HOST};
	my $addr = $ENV{REMOTE_ADDR};

	if ($cf{gethostbyaddr} && ($host eq "" || $host eq $addr)) {
		$host = gethostbyaddr(pack("C4", split(/\./, $addr)), 2);
	}

	# IPチェック
	my $flg;
	foreach ( split(/\s+/, $cf{deny_addr}) ) {
		s/\./\\\./g;
		s/\*/\.\*/g;

		if ($addr =~ /^$_/i) { $flg = 1; last; }
	}
	if ($flg) {
		&error("アクセスを許可されていません");

	# ホストチェック
	} elsif ($host) {

		foreach ( split(/\s+/, $cf{deny_host}) ) {
			s/\./\\\./g;
			s/\*/\.\*/g;

			if ($host =~ /$_$/i) { $flg = 1; last; }
		}
		if ($flg) {
			&error("アクセスを許可されていません");
		}
	}
	$host ||= $addr;
	return ($host,$addr);
}

#-----------------------------------------------------------
#  時間取得
#-----------------------------------------------------------
sub get_time {
	$ENV{TZ} = "JST-9";
	my $time = time;
	my ($min,$hour,$mday,$mon,$year,$wday) = (localtime($time))[1..6];
	my @week = qw|Sun Mon Tue Wed Thu Fri Sat|;

	# 日時のフォーマット
	my $date = sprintf("%04d/%02d/%02d(%s) %02d:%02d",
			$year+1900,$mon+1,$mday,$week[$wday],$hour,$min);

	return ($date,$time);
}

#-----------------------------------------------------------
#  crypt暗号
#-----------------------------------------------------------
sub encrypt {
	my $in = shift;

	my @wd = ('a'..'z', 'A'..'Z', 0..9, '.', '/');
	srand;
	my $salt = $wd[int(rand(@wd))] . $wd[int(rand(@wd))];
	crypt($in, $salt) || crypt ($in, '$1$' . $salt);
}

#-----------------------------------------------------------
#  crypt照合
#-----------------------------------------------------------
sub decrypt {
	my ($in, $dec) = @_;

	return 0 if (!$dec);
	my $salt = $dec =~ /^\$1\$(.*)\$/ ? $1 : substr($dec, 0, 2);
	if (crypt($in, $salt) eq $dec || crypt($in, '$1$' . $salt) eq $dec) {
		return 1;
	} else {
		return 0;
	}
}

#-----------------------------------------------------------
#  添付アップロード
#-----------------------------------------------------------
sub upload {
	my $no = shift;

	# サムネイル機能
	require './lib/thumb.pl' if ($cf{thumbnail});

	# 拡張子取得
	my $ext = $cgi->param_filename('upfile') =~ /(\.[^\\\/:\.]+)$/ ? $1 : error("拡張子不明");
	$ext =~ tr/A-Z/a-z/;
	if ($ext eq '.jpeg') { $ext = '.jpg'; }

	# ファイルチェック
	check_upl($ext);

	# 添付ファイル定義
	my $upfile = "$cf{upldir}/$no$ext";

	# アップロード書き込み
	open(UP,"+> $upfile") or error("up err: $upfile");
	binmode(UP);
	print UP $in{upfile};
	close(UP);

	chmod(0666, $upfile);

	# 画像サイズ取得
	my ($flg,$w,$h);
	if ($ext eq ".jpg") { ($w,$h) = &j_size($upfile); $flg++; }
	elsif ($ext eq ".gif") { ($w,$h) = &g_size($upfile); $flg++; }
	elsif ($ext eq ".png") { ($w,$h) = &p_size($upfile); $flg++; }

	# サムネイル化
	if ($flg && $cf{thumbnail}) {
		($w,$h) = &resize($w,$h);
		my $thumb = "$cf{upldir}/$no-s$ext";
		make_thumb($upfile,$thumb,$w,$h);
	}

	# 結果を返す
	return ($ext,$w,$h);
}

#-----------------------------------------------------------
#  JPEGサイズ認識
#-----------------------------------------------------------
sub j_size {
	my $jpg = shift;

	my ($h, $w, $t);
	open(IMG,"$jpg");
	binmode(IMG);
	read(IMG, $t, 2);
	while (1) {
		read(IMG, $t, 4);
		my ($m, $c, $l) = unpack("a a n", $t);

		if ($m ne "\xFF") {
			$w = $h = 0;
			last;
		} elsif ((ord($c) >= 0xC0) && (ord($c) <= 0xC3)) {
			read(IMG, $t, 5);
			($h, $w) = unpack("xnn", $t);
			last;
		} else {
			read(IMG, $t, ($l - 2));
		}
	}
	close(IMG);

	return ($w, $h);
}

#-----------------------------------------------------------
#  GIFサイズ認識
#-----------------------------------------------------------
sub g_size {
	my $gif = shift;

	my $data;
	open(IMG,"$gif");
	binmode(IMG);
	sysread(IMG, $data, 10);
	close(IMG);

	if ($data =~ /^GIF/) { $data = substr($data, -4); }
	my $w = unpack("v", substr($data, 0, 2));
	my $h = unpack("v", substr($data, 2, 2));

	return ($w, $h);
}

#-----------------------------------------------------------
#  PNGサイズ認識
#-----------------------------------------------------------
sub p_size {
	my $png = shift;

	my $data;
	open(IMG,"$png");
	binmode(IMG);
	read(IMG, $data, 24);
	close(IMG);

	my $w = unpack("N", substr($data, 16, 20));
	my $h = unpack("N", substr($data, 20, 24));

	return ($w, $h);
}

#-----------------------------------------------------------
#  ファイルチェック
#-----------------------------------------------------------
sub check_upl {
	my ($ext) = @_;

	# mimeタイプ
	my $mime = $cgi->param_mime('upfile');

	my $flg;
	if ($cf{ok_gif}) {
		if ($mime =~ /^image\/gif$/i and $ext eq '.gif') { $flg++; }
	}
	if (!$flg and $cf{ok_jpeg}) {
		if ($mime =~ /^image\/p?jpe?g$/i and $ext eq '.jpg') { $flg++; }
	}
	if (!$flg and $cf{ok_png}) {
		if ($mime =~ /^image\/png$/i and $ext eq 'png') { $flg++; }
	}
	if (!$flg and $cf{ok_pdf}) {
		if ($mime =~ /^application\/pdf$/i and $ext eq '.pdf') { $flg++; }
	}
	if (!$flg and $cf{ok_zip}) {
		if ($mime =~ /^application\/(x-)?zip(-compressed)?$/i and $ext eq '.zip') { $flg++; }
	}
	if (!$flg and $cf{ok_text}) {
		if ($mime =~ /^text\/plain$/i and $ext =~ /^\.te?xt$/) { $flg++; }
	}
	if (!$flg and $cf{ok_word}) {
		if ($mime =~ /^application\/(vnd\.)?ms-?word$/i and $ext eq '.doc') { $flg++; }
		elsif ($mime =~ /^application\/vnd\.openxmlformats-officedocument\.wordprocessingml\.document$/i and $ext eq '.docx') { $flg++; }
	}
	if (!$flg and $cf{ok_excel}) {
		if ($mime =~ /^application\/(vnd\.)?ms-?excel$/i and $ext eq '.xls') { $flg++; }
		elsif ($mime =~ /^application\/vnd\.openxmlformats-officedocument\.spreadsheetml\.sheet$/i and $ext eq '.xlsx') { $flg++; }
	}
	if (!$flg and $cf{ok_ppt}) {
		if ($mime =~ /^application\/(vnd\.)?ms-?powerpoint$/i and $ext eq '.ppt') { $flg++; }
		elsif ($mime =~ /^application\/vnd\.openxmlformats-officedocument\.presentationml\.presentation$/i and $ext eq '.pptx') { $flg++; }
	}
	if (!$flg and $cf{ok_mp3}) {
		if ($mime =~ /^audio\/mpeg$/i and $ext eq '.mp3') { $flg++; }
		elsif ($mime =~ /^audio\/mp3/i and $ext eq '.mp3') { $flg++; }
		elsif ($mime =~ /^audio\/mpg/i and $ext eq '.mp3') { $flg++; }
	}
	if (!$flg and $cf{ok_mp4}) {
		if ($mime =~ /^video\/mpeg$/i and $ext eq '.mp4') { $flg++; }
		elsif ($mime =~ /^video\/mp4/i and $ext eq '.mp4') { $flg++; }
		elsif ($mime =~ /^video\/mpg/i and $ext eq '.mp4') { $flg++; }
	}

	if (!$flg) { error('このファイルは取り扱いできません'); }
}

#-----------------------------------------------------------
#  フォーム入力チェック
#-----------------------------------------------------------
sub form_check {
	# 不要改行カット
	$in{sub}  =~ s/<br>//g;
	$in{name} =~ s/<br>//g;
	$in{pwd}  =~ s/<br>//g;
	$in{captcha} =~ s/<br>//g;
	$in{color} =~ s/\D//g;
	$in{icon}  =~ s/\D//g;
	$in{comment} =~ s/(<br>)+$//g;

	# チェック
	if ($cf{no_wd}) { no_wd(); }
	if ($cf{jp_wd}) { jp_wd(); }
	if ($cf{urlnum} > 0) { urlnum(); }

	# 入力値調整
	if ($in{url} eq 'http://') { $in{url} = ''; }
	$in{sub} ||= "無題";

	# 入力項目チェック
	my $err;
	if (length($in{sub}) > $cf{sub_len}*2) {
		$err .= "タイトル名は全角$cf{sub_len}文字以内です<br>";
	}
	if ($in{name} eq "") { $err .= "名前が入力されていません<br>"; }
	if ($in{comment} eq "") { $err .= "コメントが入力されていません<br>"; }
	if ($in{email} ne '' && $in{email} !~ /^[\w\.\-]+\@[\w\.\-]+\.[a-zA-Z]{2,6}$/) {
		$err .= "Ｅメールの入力内容が不正です<br>";
	}
	if ($in{url} ne '' && $in{url} !~ /^https?:\/\/[\w-.!~*'();\/?:\@&=+\$,%#]+$/) {
		$err .= "URL情報が不正です<br>";
	}
	if ($cf{my_icon} == 1 && $#{$cf{icon}} == $in{icon}) {
		if ($in{pwd} ne $cf{password}) {
			$err .= "管理者アイコンは管理者以外は選択できません<br />また管理者アイコン使用記事は管理モードにて修正します<br />";
		}
	}
	error($err) if ($err);
}

#-----------------------------------------------------------
#  クッキー発行
#-----------------------------------------------------------
sub set_cookie {
	my @data = @_;

	my ($sec,$min,$hour,$mday,$mon,$year,$wday,undef,undef) = gmtime(time + 60*24*60*60);
	my @mon  = qw|Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec|;
	my @week = qw|Sun Mon Tue Wed Thu Fri Sat|;

	# 時刻フォーマット
	my $gmt = sprintf("%s, %02d-%s-%04d %02d:%02d:%02d GMT",
				$week[$wday],$mday,$mon[$mon],$year+1900,$hour,$min,$sec);

	# URLエンコード
	my $cook;
	foreach (@data) {
		s/(\W)/sprintf("%%%02X", unpack("C", $1))/eg;
		$cook .= "$_<>";
	}

	print "Set-Cookie: $cf{cookie_id}=$cook; expires=$gmt\n";
}

#-----------------------------------------------------------
#  フッター
#-----------------------------------------------------------
sub footer {
	my $foot = shift;

	# 著作権表記（削除・改変禁止）
	my $copy = <<EOM;
<p style="margin-top:2.5em;text-align:center;font-family:Verdana,Helvetica,Arial;font-size:11pt;">
- <a href="https://www.kent-web.com/" target="_top">$cf{version}</a> -<br>
++ <a href="https://www.transformsite.com/" target="_top">Edited by Hamel</a> ++
</p>
EOM

	if ($foot =~ /(.+)(<\/body[^>]*>.*)/si) {
		print "$1$copy$2\n";
	} else {
		print "$foot$copy\n";
		print "</body></html>\n";
	}
	exit;
}

#-----------------------------------------------------------
#  文字数カウント for UTF-8
#-----------------------------------------------------------
sub count_str {
	my ($str) = @_;
	
	# UTF-8定義
	my $byte1 = '[\x00-\x7f]';
	my $byte2 = '[\xC0-\xDF][\x80-\xBF]';
	my $byte3 = '[\xE0-\xEF][\x80-\xBF]{2}';
	my $byte4 = '[\xF0-\xF7][\x80-\xBF]{3}';
	
	my $i = 0;
	while ($str =~ /($byte1|$byte2|$byte3|$byte4)/gx) {
		$i++;
	}
	return $i;
}

#-----------------------------------------------------------
#  mimeエンコード
#  [quote] http://www.din.or.jp/~ohzaki/perl.htm#JP_Base64
#-----------------------------------------------------------
sub mime_unstructured_header {
  my $oldheader = shift;
  jcode::convert(\$oldheader,'euc','utf8');
  my ($header,@words,@wordstmp,$i);
  my $crlf = $oldheader =~ /\n$/;
  $oldheader =~ s/\s+$//;
  @wordstmp = split /\s+/, $oldheader;
  for ($i = 0; $i < $#wordstmp; $i++) {
    if ($wordstmp[$i] !~ /^[\x21-\x7E]+$/ and
	$wordstmp[$i + 1] !~ /^[\x21-\x7E]+$/) {
      $wordstmp[$i + 1] = "$wordstmp[$i] $wordstmp[$i + 1]";
    } else {
      push(@words, $wordstmp[$i]);
    }
  }
  push(@words, $wordstmp[-1]);
  foreach my $word (@words) {
    if ($word =~ /^[\x21-\x7E]+$/) {
      $header =~ /(?:.*\n)*(.*)/;
      if (length($1) + length($word) > 76) {
	$header .= "\n $word";
      } else {
	$header .= $word;
      }
    } else {
      $header = add_encoded_word($word, $header);
    }
    $header =~ /(?:.*\n)*(.*)/;
    if (length($1) == 76) {
      $header .= "\n ";
    } else {
      $header .= ' ';
    }
  }
  $header =~ s/\n? $//mg;
  $crlf ? "$header\n" : $header;
}
sub add_encoded_word {
  my ($str, $line) = @_;
  my $result;
  my $ascii = '[\x00-\x7F]';
  my $twoBytes = '[\x8E\xA1-\xFE][\xA1-\xFE]';
  my $threeBytes = '\x8F[\xA1-\xFE][\xA1-\xFE]';
  while (length($str)) {
    my $target = $str;
    $str = '';
    if (length($line) + 22 +
	($target =~ /^(?:$twoBytes|$threeBytes)/o) * 8 > 76) {
      $line =~ s/[ \t\n\r]*$/\n/;
      $result .= $line;
      $line = ' ';
    }
    while (1) {
      my $encoded = '=?ISO-2022-JP?B?' .
      b64encode(jcode::jis($target,'euc','z')) . '?=';
      if (length($encoded) + length($line) > 76) {
	$target =~ s/($threeBytes|$twoBytes|$ascii)$//o;
	$str = $1 . $str;
      } else {
	$line .= $encoded;
	last;
      }
    }
  }
  $result . $line;
}
# [quote] http://www.tohoho-web.com/perl/encode.htm
sub b64encode {
    my $buf = shift;
    my ($mode,$tmp,$ret);
    my $b64 = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
                . "abcdefghijklmnopqrstuvwxyz"
                . "0123456789+/";
	
    $mode = length($buf) % 3;
    if ($mode == 1) { $buf .= "\0\0"; }
    if ($mode == 2) { $buf .= "\0"; }
    $buf =~ s/(...)/{
        $tmp = unpack("B*", $1);
        $tmp =~ s|(......)|substr($b64, ord(pack("B*", "00$1")), 1)|eg;
        $ret .= $tmp;
    }/eg;
    if ($mode == 1) { $ret =~ s/..$/==/; }
    if ($mode == 2) { $ret =~ s/.$/=/; }
    
    return $ret;
}

